;; bitey/theme.jl

;; Adam Clark <yoda@dagobah.com>

;; Based on Gradient, Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>

(require 'gradient)

;; Commentary:

(defgroup bitey "Bitey Theme"
  :group appearance)

(defcustom bitey:gradient-type 'diagonal
  "Direction of gradient in `gradient' frame style."
  :type symbol
  :options (horizontal vertical diagonal)
  :group (appearance bitey)
  :after-set after-setting-frame-option)

(defcustom bitey:normal-from-color (get-color "#c4c4c4")
  "`From' color of inactive frames in `gradient' frame style."
  :type color
  :group (appearance bitey)
  :after-set after-setting-frame-option)

(defcustom bitey:normal-to-color (get-color "#909090")
  "`To' color of inactive frames in `gradient' frame style."
  :type color
  :group (appearance bitey)
  :after-set after-setting-frame-option)

(defcustom bitey:active-from-color (get-color "#ffca00")
  "`From' color of active frames in `gradient' frame style."
  :type color
  :group (appearance bitey)
  :after-set after-setting-frame-option)

(defcustom bitey:active-to-color (get-color "#d45900")
  "`To' color of active frames in `gradient' frame style."
  :type color
  :group (appearance bitey)
  :after-set after-setting-frame-option)

(defcustom bitey:save-memory t
  "Use less memory when creating gradients, possibly affecting quality."
  :type boolean
  :group (appearance bitey)
  :after-set after-setting-frame-option)

(let*
    ((render-bg (lambda (img state)
		  (apply (cond ((eq bitey:gradient-type 'diagonal)
				draw-diagonal-gradient)
			       ((eq bitey:gradient-type 'horizontal)
				draw-horizontal-gradient)
			       ((eq bitey:gradient-type 'vertical)
				draw-vertical-gradient))
			 img (if state
				 (list bitey:active-from-color
				       bitey:active-to-color)
			       (list bitey:normal-from-color
				     bitey:normal-to-color)))
		  (when (> (cdr (image-dimensions img)) 2)
		    (bevel-image img 1 (not (eq state 'clicked))))
		  (set-image-border img 1 1 1 1)))

     (flat-bg-to (lambda (img state)
		(clear-image img (if state 
				     bitey:active-to-color
				   bitey:normal-to-color))
		(when (> (cdr (image-dimensions img)) 2)
		  (bevel-image img 1 (not (eq state 'clicked))))
		(set-image-border img 1 1 1 1)))

     (flat-bg-from (lambda (img state)
		(clear-image img (if state 
				     bitey:active-from-color
				   bitey:normal-from-color))
		(when (> (cdr (image-dimensions img)) 2)
		  (bevel-image img 1 (not (eq state 'clicked))))
		(set-image-border img 1 1 1 1)))


     (to-color (lambda (w)
		     (list bitey:active-to-color
			   bitey:normal-to-color)))

     (from-color (lambda (w)
		     (list bitey:active-from-color
			   bitey:normal-from-color)))

     (scale (lambda () (if bitey:save-memory 2 1)))

     ;; 11x11
     (max (list (make-image "max.png") nil (make-image "max_o.png") 
		(make-image "max_a.png")))
     (close (list (make-image "close.png") nil (make-image "close_o.png")
		  (make-image "close_a.png")))

     (frame `(((renderer . ,render-bg)
	       (render-scale . ,scale)
	       (foreground . "black")
	       (text . ,window-name)
	       (x-justify . 3)
	       (y-justify . center)
	       (left-edge . 6)
	       (right-edge . 6)
	       (top-edge . -18)
	       (height . 18)
	       (class . title))
	      ((renderer . ,flat-bg-from)
	       (left-edge . 0)
	       (width . 6)
	       (top-edge . -18)
	       (height . 18)
	       (class . top-left-corner))
	      ((renderer . ,flat-bg-to)
	       (right-edge . 0)
	       (width . 6)
	       (top-edge . -18)
	       (height . 18)
	       (class . top-right-corner))
	      ((background . "black")
	       (left-edge . 0)
	       (right-edge . 0)
	       (top-edge . -19)
	       (height . 1))
	      ((background . "black")
	       (left-edge . -1)
	       (width . 1)
	       (top-edge . -19)
	       (bottom-edge . -7))
	      ((background . "black")
	       (right-edge . -1)
	       (width . 1)
	       (top-edge . -19)
	       (bottom-edge . -7))
	      ((renderer . ,render-bg)
	       (render-scale . ,scale)
	       (left-edge . 18)
	       (right-edge . 18)
	       (bottom-edge . -6)
	       (height . 6)
	       (class . bottom-border))
	      ((renderer . ,flat-bg-from)
	       (left-edge . 0)
	       (width . 18)
	       (bottom-edge . -6)
	       (height . 6)
	       (class . bottom-left-corner))
	      ((renderer . ,flat-bg-to)
	       (right-edge . 0)
	       (width . 18)
	       (bottom-edge . -6)
	       (height . 6)
	       (class . bottom-right-corner))
	      ((background . "black")
	       (left-edge . 0)
	       (right-edge . 0)
	       (bottom-edge . -7)
	       (height . 1))
	      ((background . ,max)
	       (right-edge . 19)
	       (top-edge . -16)
	       (class . maximize-button)
	       (removable . t))
	      ((background . ,close)
	       (right-edge . 8)
	       (top-edge . -16)
	       (class . close-button)
	       (removable . t))))

     (shaded-frame `(((renderer . ,render-bg)
		      (render-scale . ,scale)
		      (foreground . "black")
		      (text . ,window-name)
		      (x-justify . 9 )
		      (y-justify . center)
		      (left-edge . 0)
		      (right-edge . 0)
		      (top-edge . -18)
		      (height . 18)
		      (class . title))
		     ((background . "black")
		      (left-edge . 0)
		      (right-edge . 0)
		      (top-edge . -19)
		      (height . 1))
		     ((background . "black")
		      (left-edge . -1)
		      (width . 1)
		      (top-edge . -19)
		      (height . 20))
		     ((background . "black")
		      (right-edge . -1)
		      (width . 1)
		      (top-edge . -19)
		      (height . 20))
		     ((background . "black")
		      (left-edge . 0)
		      (right-edge . 0)
		      (top-edge . -1)
		      (height . 1))
		      ((background . ,max)
		       (right-edge . 19)
		       (top-edge . -16)
		       (class . maximize-button)
		       (removable . t))
		      ((background . ,close)
		       (right-edge . 8)
		       (top-edge . -16)
		       (class . close-button)
		       (removable . t))))

     (transient-frame `(((renderer . ,render-bg)
			 (render-scale . ,scale)
			 (left-edge . 0)
			 (right-edge . 0)
			 (top-edge . -5)
			 (height . 5)
			 (class . title))
			((background . "black")
			 (left-edge . 0)
			 (right-edge . 0)
			 (top-edge . -6)
			 (height . 1))
			((background . "black")
			 (left-edge . -1)
			 (width . 1)
			 (top-edge . -6)
			 (bottom-edge . -5))
			((background . "black")
			 (right-edge . -1)
			 (width . 1)
			 (top-edge . -6)
			 (bottom-edge . -5))
			((renderer . ,render-bg)
			 (render-scale . ,scale)
			 (left-edge . 0)
			 (right-edge . 0)
			 (bottom-edge . -4)
			 (height . 4)
			 (class . bottom-border))
			((background . "black")
			 (left-edge . 0)
			 (right-edge . 0)
			 (bottom-edge . -5)
			 (height . 1))))

     (shaded-transient-frame `(((renderer . ,render-bg)
				(render-scale . ,scale)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -6)
				(height . 5)
				(class . title))
			       ((background . "black")
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -7)
				(height . 1))
			       ((background . "black")
				(left-edge . -1)
				(width . 1)
				(top-edge . -7)
				(height . 7))
			       ((background . "black")
				(right-edge . -1)
				(width . 1)
				(top-edge . -7)
				(height . 7))
			       ((background . "black")
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -1)
				(height . 1)))))

  (add-frame-style 'bitey
    (lambda (w type)
      (cond 
            ((eq type 'shaped) shaded-frame)
            ((eq type 'shaded) shaded-frame)
            ((eq type 'transient) transient-frame)
            ((eq type 'shaped-transient) shaded-transient-frame)
            ((eq type 'shaded-transient) shaded-transient-frame)
            ((eq type 'unframed) nil-frame)
            (t frame))))

)
